/** \file chip45_commands.h a bunch of kcmd command definitions for
 * use on the ATMEGA644P.
 */
#include "../k644p.h"


//typedef struct struct_ADC_DATA
//{
//   uint8_t m_index;  // counts up with each sample, modulo N_ADC_FILTERPT
//   uint8_t m_chan;   // goes from 0 to N_ADC_CHAN
//   uint16_t m_rawdata[N_ADC_ARRAYSZ];
//   uint16_t m_avgdata[N_ADC_ARRAYSZ];
//   uint16_t m_nConv; // Keep track of the number of conversions for fun
//} ADC_DATA;

void kcmd_printStatus(KCMD *pKCMD)
{
  ADC_DATA *pAdc;
  uint8_t iChan;
  uint8_t index;
  uint16_t nConv;
//  uint16_t data[4];
   
  pAdc=getAdcDataPtr();

  kcmd_Printf_1(pKCMD, KCCSTR("ptr to ADC struct: %p"), pAdc);
  /* Halt interrupts while we gather the data we need from the struct. */
  cli();
  iChan=pAdc->m_chan;
  index=pAdc->m_index;
  nConv = pAdc->m_nConv;
  sei();
  kcmd_Printf_3(pKCMD, KCCSTR("\r\niChan: %d idx: %d nConv: %d"),
    iChan, index, nConv);
}


MAKE_KCMD_COMMAND( printStatus,  kcmd_printStatus, "[] -- [] types out ADC status", g_str_system );
#undef PREVNAME
#define PREVNAME G_ME(printStatus)

void kcmd_spewAdc(KCMD *pKCMD)
{
   ADC_DATA *pAdc;
   uint16_t adcvalues[N_ADC_CHAN];
   pAdc=getAdcDataPtr();
   uint16_t iConv;

   do
   {
      /* Halt interrupts while we gather the data we need from the struct. */
      cli();
      adcvalues[0] = pAdc->m_rawdata[0];
	  adcvalues[1] = pAdc->m_rawdata[1 << ADC_FILTERSHIFT];
	  adcvalues[2] = pAdc->m_rawdata[2 << ADC_FILTERSHIFT];
	  iConv = pAdc->m_nConv;
	  sei();
	  if( !(iConv & ADC_FILTERMASK))
	  {
        kcmd_Printf_3(pKCMD, KCCSTR("0x%04x, 0x%04x, 0x%04x\r\n"), adcvalues[0], adcvalues[1], adcvalues[2]);
	  }
   } while(!pKCMD->pUserIO->pFuncKbhit(pKCMD->conio_context) );
}

MAKE_KCMD_COMMAND( spewAdc, kcmd_spewAdc, "[] -- [] Spew Adc data until kbhit", g_str_system);
#undef PREVNAME
#define PREVNAME G_ME(spewAdc)


void kcmd_spewAvgAdc(KCMD *pKCMD)
{
   ADC_DATA *pAdc;
   uint16_t adcvalues[N_ADC_CHAN];
   pAdc=getAdcDataPtr();
   uint16_t iConv;

   do
   {
      /* Halt interrupts while we gather the data we need from the struct. */
      cli();
      adcvalues[0] = pAdc->m_avgdata[0];
	  adcvalues[1] = pAdc->m_avgdata[1 << ADC_FILTERSHIFT];
	  adcvalues[2] = pAdc->m_avgdata[2 << ADC_FILTERSHIFT];
	  iConv = pAdc->m_nConv;
	  sei();
	  if( !(iConv & ADC_FILTERMASK))
	  {
        kcmd_Printf_3(pKCMD, KCCSTR("0x%04x, 0x%04x, 0x%04x\r\n"), adcvalues[0], adcvalues[1], adcvalues[2]);
	  }
   } while(!pKCMD->pUserIO->pFuncKbhit(pKCMD->conio_context) );
}

MAKE_KCMD_COMMAND( spewAvgAdc, kcmd_spewAvgAdc, "[] -- [] Spew Adc data until kbhit", g_str_system);
#undef PREVNAME
#define PREVNAME G_ME(spewAvgAdc)
