/** \file k644p.h  defines for the ATMEGA644p Perhipheral routines
 *
 */
#ifndef K644P_H__
#define K644P_H__


/** Number of real time clock ticks in one second, .992 ms each */
#define TMR0_TICKS_PER_SEC 1008

/**
 * The number of ADC samples per second.
 */
#define ADC_SAMPLE_RATE 240


/**
 * TIMERSTATE_t is a container for the variables which make up
 * the state of a timer.
 */
typedef struct struct_TIMERSTATE_t
{
  volatile uint16_t tickCount;  
  volatile uint16_t secondCount;
  volatile uint16_t  delayCounter;
} TIMERSTATE_t;



/**
 * N_ADC_CHAN is the number of ADC channels we are collecting from. 
 */
#define N_ADC_CHAN  3

/**
 * N_ADC_FILTERPT is the number of samples in a 60hz filter removal scheme
 */
#define N_ADC_FILTERPT 4

/**
 * ADC_FILTERMASK is the value to 'AND' the index with to cycle 0-N_ADC_FILTERPT
 */
#define ADC_FILTERMASK 0x3

/**
 * ADC_FILTERSHIFT is the power of 2 of N_ADC_FILTERPT 
 */
#define ADC_FILTERSHIFT 2

/**
 * N_ADC_ARRAYSZ is the size of the sample arrays needed for filtering.
 */
#define N_ADC_ARRAYSZ (N_ADC_CHAN * N_ADC_FILTERPT)

/**
 * N_ADC_POW_SHIFTDIV is the number of bits to shift the difference down.
 *  in order to time average. 5 = shift down by 5 bits, or (divides by 32)
 */
#define N_ADC_POW_SHIFTDIV 4  // Divide difference by 16

/**
 * N_ADC_POWDIV is the number of bits we shift the avg down before subtracting
 * it from the current ADC value. 
 */
typedef struct struct_ADC_DATA
{
   uint8_t m_index;  // counts up with each sample, modulo N_ADC_FILTERPT
   uint8_t m_chan;   // goes from 0 to N_ADC_CHAN
   uint16_t m_rawdata[N_ADC_ARRAYSZ];
   uint16_t m_avgdata[N_ADC_ARRAYSZ];
   uint16_t m_nConv; // Keep track of the number of conversions for fun
} ADC_DATA;



/**
 * get_tmr0_ticks() gets the current value of the tmr0 tick counter.
 */
extern uint16_t get_tmr0_ticks(void);


/**
 * initTimer0() initializes timer0
 */
extern void initTimer0(void);

/**
 * initTimer1() initializes timer1
 */
extern void initTimer1(void);

/**
 * initADC() intializes the ADC and the data structure
 */
void initADC(void);

/**
 * initGpio() initalizes the GPIO lines.
 */
void initGpio(void);

/**
 * getAdcDataPtr() retrieves the current ADC data structure.
 */
ADC_DATA *getAdcDataPtr(void);

/**
 * wait_msec() waits for a nMsec to elapse before returning.
 */
extern void wait_msec(uint16_t nMsec);

#endif /* K644P_H__ */
