/** \file my_commands.h is the include for the commands module. 
 *
 * This include is
 * where other include files with more commands can be easily added.  Since this 
 * include file compiles code into global variables, it is NOT to be included more
 * than once in different C modules.  Instead, the main function of the user should
 * include this file after including kcmdIncludes.h
 */
#ifndef MY_COMMANDS_H__
#define MY_COMMANDS_H__

/**
 * Include the file with the macros for defining commands in it.
 */
#include "kcmd/kcmdCommandMacros.h"

/**
 * Make sure this is not defined. 
 */
#undef PREVNAME

/**
 * This macro is redefined to be the previously defined command.  It is used
 * for having the compile construct a linked list of global static structures
 * for us.  It is defined as g_kcmd_help because that will be the first command
 * in the list.  This is the command already defined in kcmd.c.  If this doesn't
 * make any sense to you, don't worry about it.  PREVNAME is just part of the 
 * trick to get the C preprocessor to make a linked list of commands in program
 * memory for us.  Just follow the doc for making
 * new commands and everything probably will work just fine.
 */
#define PREVNAME g_kcmd_bye 

/**
 * kcmd_basic_commands is a file with the basic commands in it.
 */
#include "kcmd/commands/kcmd_basic_commands.h"


/****************************************************************************
 * Here is where more include files with more commands would be added.   
 * Simply follow the model of kcmd_basic_commands.
 ****************************************************************************/

//#include "my_custom_commands.h"
#include "k644p/commands/chip45_commands.h"


/****************************************************************************
 * Here we define the last command in the list.  It is important that this
 * stay at the END of this file, because it is only command which has a 
 * pNext == NULL.
 ****************************************************************************/

#include "kcmd/commands/kcmd_first_command.h"

#endif /* MY_COMMANDS_H__ */
