/** \file my_commands.h is the include for the commands module. 
 *
 * This include file should be modified and placed in your project, since each
 * project might have some different content in it.  
 * 
 * NOTE:  ONLY INCLUDE THIS FILE ***ONCE*** in your main source file.  DO NOT
 * include this file from more than one source file or you will get compile
 * errors from having multiply-defined global variables.
 *
 * Instead, the main function of the user should only include this file
 * from their main source after kcmdIncludes.h
 */
#ifndef MY_COMMANDS_H__
#define MY_COMMANDS_H__

/**
 * Include the file with the macros for defining commands in it.
 */
#include "kcmdCommandMacros.h"

/**
 * Make sure this is not defined. 
 */
#undef PREVNAME

/**
 * This macro is redefined to be the previously defined command.  It is used
 * for having the compile construct a linked list of global static structures
 * for us.  It is defined as g_kcmd_help because that will be the first command
 * in the list.  This is the command already defined in kcmd.c.  If this doesn't
 * make any sense to you, don't worry about it.  PREVNAME is just part of the 
 * trick to get the C preprocessor to make a linked list of commands in program
 * memory for us.  Just follow the doc for making
 * new commands and everything probably will work just fine.
 */
#define PREVNAME g_kcmd_bye 

/**
 * kcmd_basic_commands is a file with the basic commands in it.
 */
#include "commands/kcmd_basic_commands.h"


/****************************************************************************
 * Here is where more include files with more commands would be added.   
 * Simply follow the model of kcmd_basic_commands.
 ****************************************************************************/

//#include "my_custom_commands.h"


/****************************************************************************
 * Here we define the last command in the list.  It is important that this
 * stay at the END of this file, because it is only command which has a 
 * pNext == NULL.
 ****************************************************************************/

#include "commands/kcmd_first_command.h"

#endif /* MY_COMMANDS_H__ */
