/*
	kcmd command processor - Copyright (C) 2009 Kevin Stokes.

	This file is part of the kcmd command processor distribution
	originally availabe from http://www.boldinventions.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the Eclipse Public License - v 1.0.

	This software has NO WARRANTY.  Use it at your own risk;   It 
	has no warranty of any kind, implied or otherwise.

	There should be a copy of the Eclipse Public License along with
	this source code.  If not, it can be found on the eclipse.org
        website:  http://www.eclipse.org

*/

/** \file kcmd.h is the header file for the portable parts of the kcmd cmd processor
 *
 */
#ifndef KCMD_H
#define KCMD_H


/*
 * Define some flag bits for return values from the process command line function. 
 */
#define KCMD_FLAG_EXIT 1
#define KCMD_RFLAG_USEREXIT 1
#define KCMD_RFLAG_BADCOMMAND 2
#define KCMD_RFLAG_IGNOREREST 4
#define KCMD_RFLAG_NOINPUT 8
#define KCMD_RFLAG_STACKOVERFLOW 16
#define KCMD_RFLAG_STACKUNDERFLOW 32


/* If we are in a c++ environment, make sure the compiler knows these are */
/* straight C functions. */
#ifdef __cplusplus
extern "C" {
#endif


/**
 * KCMD is the structure for the container which holds the command processor's
 * state.  Almost all the kcmd functions are passed a pointer to this structure.
 *   It holds pointers to the IO routines, the argument stack for commands,
 * the current prompt string, and the input command line.
 */
typedef struct kcmd_struct
{
   int flags;
   int base;
   KINTPTR_t stack[N_CMDVM_STACK];
   int iStack;
   int inputTimeoutInMS;
   USER_IO *pUserIO;
   void *conio_context;
   KCHAR *pCmdLine;
   KCCHAR *pPromptString;
   KCCHAR *pWelcomeString;
} KCMD;


/**
 * FUNC_CONSOLE_CMD is the function typedef for kcmd command functions.
 * just make an instance of a function of this type, and add it to the 
 * current list of commands with kcmd_CreateNewCmd()
 */
typedef void (*FUNC_CONSOLE_CMD)(KCMD *pKCMD);


/**
 * The structure of the element in the list of all defined commands.
 */
typedef struct KCMD_CMD_LIST_ELEM_struct
{
   const struct KCMD_CMD_LIST_ELEM_struct *pNext;  /* Link to next command */
   KCCHAR *pcmdName;              /* ptr to Command Name */
   FUNC_CONSOLE_CMD pFunkcmd_;        /* ptr to C function */
   KCCHAR *pHelpText;             /* ptr to Help function (can be NULL) */
   KCCHAR *pCategory;          /* ptr to Category name string */
} KCMD_CMD_LIST_ELEM;



/*
 * Some category names. 
 */
extern KCCHAR g_str_system[];

/* Prototype for the kcmd_Help command. */
void kcmd_Help(KCMD *pKCMD);



/* Define the first command to show up in the linked list. */
extern const KCMD_CMD_LIST_ELEM KCSTRUCTATTR g_kcmd_help;

/* Define the first command to show up in the linked list. */
extern const KCMD_CMD_LIST_ELEM KCSTRUCTATTR g_kcmd_bye;

/**
 * kcmd_cmdInit is the low level init function.  It returns a ptr to a KCMD structure
 * to hold the current state of the command processor.
 */
void *kcmd_cmdInit(USER_IO *pUserIO, 
      void *conio_context, 
	  KCCHAR *promptString,
	  KCCHAR *pWelcomeString, 
	  int inputTimeout);

/**
 * kcmd_cmdExec is called to parse a line of input from the input device and
 * execute all the commands on that line.
 */
int kcmd_cmdExec(void *cmd_context, int bPrintPrompt);

/**
 * kcmd_cmdClose
 */
void kcmd_cmdClose(void *cmd_context);

#ifdef __cplusplus
} /* extern "C" ... */
#endif

#endif /* KCMD_H */
