/*
	kcmd command processor - Copyright (C) 2009 Kevin Stokes.

	This file is part of the kcmd command processor distribution
	originally availabe from http://www.boldinventions.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the Eclipse Public License - v 1.0.

	This software has NO WARRANTY.  Use it at your own risk;   It 
	has no warranty of any kind, implied or otherwise.

	There should be a copy of the Eclipse Public License along with
	this source code.  If not, it can be found on the eclipse.org
        website:  http://www.eclipse.org

*/

/** \file kcmd_command_macros.h contains macros for simplifying command creation
 *
 */
#ifndef KCMD_COMMAND_MACROS_H__
#define KCMD_COMMAND_MACROS_H__


/**
 * QUOTEME(x) is a macro which replaces x with "x"
 */
#define QUOTEME(x) KT(#x)

/**
 * G_ME(x) is a macro which replaces x with g_x,  as in G_ME(helptext) would
 * expand to g_helptext
 */
#define G_ME(name) g_##name

/**
 * G_STR_ME(name) expands to g_str_name
 */
#define G_STR_ME(name) g_str_##name


/**
 * G_HELPTEXT_ME(name) expands to g_helptext_name
 */
#define G_HELPTEXT_ME(name) g_helptext_##name

/**
 * Make sure this is not defined. 
 */
#undef PREVNAME

/**
 * This macro is redefined to be the previously defined command.  It is used
 * for having the compile construct a linked list of global static structures
 * for us.  It is defined as g_kcmd_help because that will be the first command
 * in the list.  This is the command already defined in kcmd.c.
 */
#define PREVNAME g_kcmd_bye 


/**
 * MAKE_KCMD_COMMAND is a macro which creates the command string,
 * the help string, and the list element in global memory.  On systems which
 * support it, the strings and the list element itself will be stored in 
 * Flash program memory rather than RAM.
 *   This macro is meant to be called in a sequence which redefines PREVNAME
 * so that the preprocessor can construct a linked list as each command is 
 * added. 
 */
#define MAKE_KCMD_COMMAND( name, func_name, helptext, category ) \
KCCHAR KCCATTR G_STR_ME(name)[] = QUOTEME(name); \
KCCHAR KCCATTR G_HELPTEXT_ME(name)[] = KT(helptext); \
const KCMD_CMD_LIST_ELEM KCSTRUCTATTR G_ME(name) = \
{ &PREVNAME, G_STR_ME(name), func_name, G_HELPTEXT_ME(name), category }

/**
 * MAKE_KCMD_COMMAND_5 is a macro very similar to MAKE_KCMD_COMMAND, except
 * it is possible to make the name of the command be different than 
 * the symbol names in the source.  For example, '+' is a good name for the
 * addition command name, but you can't have that character in the source 
 * code as part of a variable name. 
 */
#define MAKE_KCMD_COMMAND_5( name, cmdname, func_name, helptext, category ) \
KCCHAR KCCATTR G_STR_ME(name)[] = KT(cmdname); \
KCCHAR KCCATTR G_HELPTEXT_ME(name)[] = KT(helptext); \
const KCMD_CMD_LIST_ELEM KCSTRUCTATTR G_ME(name) = \
{ &PREVNAME, G_STR_ME(name), func_name, G_HELPTEXT_ME(name), category }

#endif /* KCMD_COMMAND_MACROS_H__ */

