/*
	kcmd command processor - Copyright (C) 2009 Kevin Stokes.

	This file is part of the kcmd command processor distribution
	originally availabe from http://www.boldinventions.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the Eclipse Public License - v 1.0.

	This software has NO WARRANTY.  Use it at your own risk;   It 
	has no warranty of any kind, implied or otherwise.

	There should be a copy of the Eclipse Public License along with
	this source code.  If not, it can be found on the eclipse.org
        website:  http://www.eclipse.org

*/

/** \file kcmd_helper.c has some general helper functions
  *
  */
#include "kcmdIncludes.h"

/**
 * This function converts a string into a number.  Returns the converted value.  If piErr!=null,
 * returns an error code to report if the conversion worked.  Negative values mean the conversion failed.
 **/
int str_to_num(KCHAR * str, int *piErr )
{
   int iRet;
   int iErr=0;
   KCHAR *strEndConv;
   
   iRet= (int) strtol(str, &strEndConv, 0);
   if(strEndConv==str)
   {
           iErr=-1;  // Error, could not covert.
   }

   if(NULL!=piErr) *piErr=iErr;
   return iRet;
}


KINTPTR_t kcmd_stackPop(KCMD *pKCMD)
{
	KINTPTR_t iRet;
        iRet=0;
        if(0<pKCMD->iStack)
	{
           pKCMD->iStack = pKCMD->iStack - 1;
           iRet=pKCMD->stack[pKCMD->iStack];
	} else
	{
           pKCMD->flags=KCMD_RFLAG_STACKUNDERFLOW;
	}
        return iRet;
}

void kcmd_stackPush(KCMD *pKCMD, int iValue)
{
   if(N_CMDVM_STACK > pKCMD->iStack) // Check for stack overflow
   {
      pKCMD->stack[pKCMD->iStack]=iValue;
      pKCMD->iStack = pKCMD->iStack+1;
   } else
   {
           pKCMD->flags=KCMD_RFLAG_STACKOVERFLOW;
           pKCMD->iStack=0;
   }
}


void kcmd_stackDup(KCMD *pKCMD)
{
   if( (pKCMD->iStack<1) || (pKCMD->iStack >= (N_CMDVM_STACK-1) ) )
   {
     pKCMD->flags=KCMD_RFLAG_STACKOVERFLOW;
      pKCMD->iStack=0;
      return;
   }
   pKCMD->stack[pKCMD->iStack]=pKCMD->stack[pKCMD->iStack-1];
   pKCMD->iStack+=1;
}

void kcmd_stackOver(KCMD *pKCMD)
{
   if( (pKCMD->iStack<2) || (pKCMD->iStack >= (N_CMDVM_STACK-1)) )
   {
     pKCMD->flags=KCMD_RFLAG_STACKOVERFLOW;
     pKCMD->iStack=0;
      return;
   }
   pKCMD->stack[pKCMD->iStack]=pKCMD->stack[pKCMD->iStack-2];
   pKCMD->iStack+=1;
}

void kcmd_stackSwap(KCMD *pKCMD)
{
   KINTPTR_t itmp;
   if( pKCMD->iStack<2) 
   {
     pKCMD->flags=KCMD_RFLAG_STACKUNDERFLOW;
     pKCMD->iStack=0;
      return;
   }
   itmp=pKCMD->stack[pKCMD->iStack-2];
   pKCMD->stack[pKCMD->iStack-2]=pKCMD->stack[pKCMD->iStack-1];
   pKCMD->stack[pKCMD->iStack-1]=itmp;
}

int kcmd_stackAt(KCMD *pKCMD)
{
  int iRet;
  int *ptr;
  
        iRet=0;
        if(0<pKCMD->iStack)
  {
           pKCMD->iStack = pKCMD->iStack - 1;
           ptr=(int *) pKCMD->stack[pKCMD->iStack];
           iRet=*ptr;
  } else
  {
           pKCMD->flags=KCMD_RFLAG_STACKUNDERFLOW;
  }
  return iRet;
}
int kcmd_stackQuestion(KCMD *pKCMD)
{
  int iRet;
  int *ptr;
  KCHAR str[32];
  iRet=0;
  if(0<pKCMD->iStack)
  {
           pKCMD->iStack = pKCMD->iStack - 1;
           ptr=(int *) pKCMD->stack[pKCMD->iStack];
           iRet=*ptr;
           my_sprintf(str, KCCSTR(" %d (0x%0x) "), iRet, iRet);
           pKCMD->pUserIO->pFuncPuts(pKCMD->conio_context, str );
  } else
  {
           pKCMD->flags=KCMD_RFLAG_STACKUNDERFLOW;
  }
  return iRet;
}



KCHAR nib_to_char(int inib)
{
        if(inib<0xa) return '0'+inib;
        return 'a'+ (inib-0xa);
}





/**
 * convert the integer argument to a text representation, sending 
 * characters to an output function callback.
 */
void hex_to_string(FUNC_CHOUT pFuncCHOUT, void *pContextCHOUT, 
                int flag_leadingzero, int flag_width,  KINTPTR_t i_arg)
{
    KCHAR cbuf[10];
    int i;
    KINTPTR_t inib;
    int ihigh;
    KINTPTR_t ileft;
    int istart;
    
//    if(flag_width==0) flag_width=8;
    if(flag_width>8) flag_width=8;
    ihigh=0;
    ileft=i_arg;
    for(i=0; i<8; i++)
    {
       inib= 0xf & ileft;
       if(inib>0 ) ihigh=i;
       cbuf[i]=nib_to_char( (int) inib);
       ileft=ileft >> 4;
    }

    /* Check for case where number does not fit into specified width. */
    if( (flag_width>0) && (ihigh >= flag_width) )
    {
       for(i=0; i<flag_width; i++) pFuncCHOUT(pContextCHOUT, '*' );
       return;
    }

    istart=ihigh;
    if( (flag_width>0) && (flag_width-1 > ihigh)) istart=flag_width-1;

    for(i=istart; i>=0; i--)
    {
       if( (i>0) && (!flag_leadingzero) &&  (i>=ihigh) && (cbuf[i]=='0') ) 
       {
          pFuncCHOUT(pContextCHOUT, ' ');
       } else 
       {
          pFuncCHOUT(pContextCHOUT, cbuf[i]);
       }
    }
}



/**
 * int_to_string() translates an integer into a sequence of 
 * characters which get written out via the supplied output
 * function callback.  This callback might just put the characters
 * in a buffer, or might go directly to the output device.
 */
void int_to_string(FUNC_CHOUT pFuncCHOUT, void *pContextCHOUT, 
                int flag_leadingzero, int flag_width,  int i_arg)
{
    KCHAR cbuf[20];
    int i;
    int inib;
    int ihigh;
    int ileft;
    int ineg=0;
    int istart;
    
//    if(flag_width==0) flag_width=11;
    if(flag_width>11) flag_width=11;
    ihigh=0;
    ileft=i_arg;
    if(ileft<0) 
    {
       ileft=0-i_arg;
       ineg=-1;
    }

    for(i=0; i<13; i++) cbuf[i]='0';

    for(i=0; i<13; i++)
    {
       inib= ileft % 10;
       cbuf[i]='0'+inib;
       if(inib>0) ihigh=i;
       ileft=ileft/10;
    }

    if(ineg)
    {
       cbuf[ihigh+1]='-';
       ihigh++;
    }

    /* Check for case where number does not fit into specified width. */
    if( (flag_width>0) && (ihigh >= flag_width) )
    {
       for(i=0; i<flag_width; i++)
       {
           pFuncCHOUT( pContextCHOUT, '*');
       }
       return;
    }

    istart=ihigh;
    if( (flag_width>0) && (flag_width-1 > ihigh)) istart=flag_width-1;

    for(i=istart; i>=0; i--)
    {
       if( (i>0) && (!flag_leadingzero) &&  (i>=ihigh) && (cbuf[i]=='0') ) 
       {
          pFuncCHOUT(pContextCHOUT, ' ');
       } else 
       {
           pFuncCHOUT(pContextCHOUT, cbuf[i]);
       }
    }
}

/**
 * string_to_string is a somewhat silly text copying function which
 * only exists for consistency with int_to_string() or hex_to_string()
 * If the input length is greater than 80 characters, it stops copying
 * at 79.
 */
void string_to_string(FUNC_CHOUT pFuncCHOUT, void *pContextCHOUT, KCHAR *str_arg)
{
    int ilen;
    int i;

    ilen=(int) strlen(str_arg);
    if( (ilen>0) && (ilen<79) ) 
    {
       for(i=0;i<ilen;i++)
       {
           pFuncCHOUT(pContextCHOUT, str_arg[i]);
       }
    }
}



void charBufferCHOUT(void *pContext, KCHAR ch)
{
   charBufferCHOUT_t *pCB = (charBufferCHOUT_t *) pContext;
   if(pCB->bufferIndex <= (pCB->bufferSize-1) )
   {
       pCB->pBuffer[pCB->bufferIndex++]=ch;
       pCB->pBuffer[pCB->bufferIndex]=0;   // Null terminate current string.
   }
}

/**
 *  my_sprintf provides some of the functionality of sprintf.  The format specifiers
 *  x, d, p, and s are implemented, and the width parameter between the percent sign
 *  and the specifier is support, as well as leading-zero.  No floating point support.
 */
KCHAR *my_sprintf(KCHAR *buf, KCCHAR *fmt, ...)
{
  va_list(ap);
  int ifmt;
  int ilen;
  KCHAR *pch;
  KCHAR ch;
//  KCHAR *pouch;
  charBufferCHOUT_t CB;
  int flag_leadingzero;
  int flag_width;
  int done;
  int i_arg;
  KINTPTR_t i_p_arg;
  void *ptr_arg;
  KCHAR *str_arg;


  va_start(ap, fmt);
  ilen=(int) KCC_strlen(fmt);
  pch=(KCHAR *) fmt;
  CB.pBuffer=buf;
  CB.bufferIndex=0;
  CB.bufferSize= nLINEBUF;
//  pouch=buf;

  for(ifmt=0; ifmt<ilen; ifmt++)
  {
     ch=KCC_ACCESS( pch++ );
//     ch=*pch++; 
     if(!ch) break;
     if( ch=='%' ) 
     {
       done=0;
       flag_leadingzero=0;
       flag_width=0;

       /* Check for '0' or numerical width parameter after % char */
       while( !done )
       {
          done=1;
		  ch=KCC_ACCESS( pch++ );
//          ch=*pch++;
          if(ch=='0') 
          {
                  flag_leadingzero=1; 
                  done=0;
          }
          if( (ch>'0') && (ch<='9') )
          {
                  flag_width= (int) (ch - '0');
                  done=0;
          }
       }
       switch (ch)
       {
          case 'x':
             i_arg=va_arg(ap, int);
             hex_to_string(charBufferCHOUT, (void *) &CB,  flag_leadingzero, flag_width,  i_arg);
          break;
          case 'd':
             i_arg=va_arg(ap, int);
             int_to_string(charBufferCHOUT, (void *) &CB, flag_leadingzero, flag_width,  i_arg);
          break;
          case 'p':
             ptr_arg=va_arg(ap, void *);
             i_p_arg=(KINTPTR_t) ptr_arg;
             charBufferCHOUT( (void *) &CB, '0');
             charBufferCHOUT( (void *) &CB, 'x');
             flag_leadingzero=1;
             hex_to_string(charBufferCHOUT, (void *) &CB, flag_leadingzero, flag_width,   i_p_arg);
          break;
          case 's':
             str_arg=va_arg(ap, KCHAR *);
             string_to_string(charBufferCHOUT, (void *) &CB, str_arg);
          break;
          case '%':
          break;
          default:
             charBufferCHOUT( (void *) &CB, '%');
             charBufferCHOUT( (void *) &CB, ch);
          break;
       }
     } else
     {
             charBufferCHOUT( (void *) &CB, ch);
     }
  }
  va_end(ap);

  return  buf;
}


/**
 * kcmdCHOUT is a very simple character out callback for sending
 * characters directly to the user instead of through a buffer.
 */
void kcmdCHOUT( void *pContext, KCHAR ch )
{
    KCMD *pKCMD=(KCMD *) pContext;
    pKCMD->pUserIO->pFuncPutc(pKCMD->conio_context, ch, 100); 
}

/**
 *  my_sprintf provides some of the functionality of sprintf.  The format specifiers
 *  x, d, p, and s are implemented, and the width parameter between the percent sign
 *  and the specifier is support, as well as leading-zero.  No floating point support.
 */
void my_kprintf(KCMD *pKCMD, KCCHAR *fmt, ...)
{
  va_list(ap);
  int ifmt;
  int ilen;
  KCHAR *pch;
  KCHAR ch;
  int flag_leadingzero;
  int flag_width;
  int done;
  int i_arg;
  KINTPTR_t i_p_arg;
  void *ptr_arg;
  KCHAR *str_arg;


  va_start(ap, fmt);
  ilen=(int) KCC_strlen(fmt);
  pch=(KCHAR *) fmt;


  for(ifmt=0; ifmt<ilen; ifmt++)
  {
     ch=KCC_ACCESS( pch++ );
//     ch=*pch++; 
     if(!ch) break;
     if( ch=='%' ) 
     {
       done=0;
       flag_leadingzero=0;
       flag_width=0;

       /* Check for '0' or numerical width parameter after % char */
       while( !done )
       {
          done=1;
		  ch=KCC_ACCESS( pch++ );
//          ch=*pch++;
          if(ch=='0') 
          {
                  flag_leadingzero=1; 
                  done=0;
          }
          if( (ch>'0') && (ch<='9') )
          {
                  flag_width= (int) (ch - '0');
                  done=0;
          }
       }
       switch (ch)
       {
          case 'x':
             i_arg=va_arg(ap, int);
             hex_to_string(kcmdCHOUT, (void *) pKCMD,  flag_leadingzero, flag_width,  i_arg);
          break;
          case 'd':
             i_arg=va_arg(ap, int);
             int_to_string(kcmdCHOUT, (void *) pKCMD, flag_leadingzero, flag_width,  i_arg);
          break;
          case 'p':
             ptr_arg=va_arg(ap, void *);
             i_p_arg=(KINTPTR_t) ptr_arg;
             kcmdCHOUT( (void *) pKCMD, '0');
             kcmdCHOUT( (void *) pKCMD, 'x');
             flag_leadingzero=1;
             hex_to_string(kcmdCHOUT, (void *) pKCMD, flag_leadingzero, flag_width,   i_p_arg);
          break;
          case 's':
             str_arg=va_arg(ap, KCHAR *);
             string_to_string(kcmdCHOUT, (void *) pKCMD, str_arg);
          break;
          case '%':
          break;
          default:
             kcmdCHOUT( (void *) pKCMD, '%');
             kcmdCHOUT( (void *) pKCMD, ch);
          break;
       }
     } else
     {
             kcmdCHOUT( (void *) pKCMD, ch);
     }
  }
  va_end(ap);
}



const KCHAR g_delimiters[]=KT(" ");


/**************************************************************************/
/**                                                                       *
 * Parse a word from a line of text.  Returns a pointer to the rest of    *
 * of the line after the parse occurs.                                    *
 * Call with the text to be parsed in   source, and provide a buffer at   *
 * least as long as  source and pass a pointer to it in token.  The       *
 * buffer you provide for token will be where your parsed word will be.   *
 * The token will be delimited by spaces or CR or LF characters.          *
 * The call specifies the size of the buffer required for the token       *
 * via the *pnTokenBufSZ argument.  If the caller needs to know how big   *
 * the buffer should be, then calling this function with *pnTokenBufSZ=0  *
 * will cause the parser to figure out how large the token will be, and   *
 * return without attempting to write to the token buffer, and            *
 * *pnTokenBufSZ will be written with the size needed to store the token. *
 * The caller can then call back, this time with *pnTokenBufSZ set to the *
 * indicated size.  The token parameter should also now be a buffer large *
 * enough to accept the parsed token.                                     *
 * The function returns a pointer to the rest of the string               *
 * which can be parsed again for the next word.                           *
 * Will return NULL if there is no more text on the line to parse after   *
 * the current word.  If there was no word at all, strlen(token)==0       *
 *   
 ****************************************************************************
 * Example for a platform with lots of RAM:
 ****************************************************************************

   char srcbuf[256];
   char token[256];
   char *rest;
   unsigned int uiLen;


   rest=srcbuf;
   while( rest!=NULL ) 
   {
     uiLen=256;
     rest=parse_text_line(rest, token, &uiLen );
     if(strlen(token)) {
	   printf("nextword: '%s'\n", token);
	 }
   }

   **************************************************************************
   Example for a thriftier platform:
   **************************************************************************
   char srcbuf[64];
   char *token;
   char *rest;
   unsigned int uiLen;

   rest=srcbuf;
   while(rest!=NULL)
   {
      // first, get the length for the token into uiLen
      uiLen=0;
      if(NULL != parse_text_line(rest, NULL, &uiLen);
      {
         // Now allocate some space to hold it.
         token=malloc(sizeof(char) * uiLen);
         if(NULL!=token)
         {
            // Now actually get the bytes of the token.
            rest = parse_text_line(rest, token, &uiLen);
            if(strlen(token))
            {
               printf("nextword: '%s'\n", token);
            }
         } // if(NULL != token
      } // if(NULL != parse...
      else break;
   }
 *                                                                       *
 **************************************************************************/
KCHAR *parse_text_line(KCHAR *source, KCHAR *token, unsigned int *pnTokenBufSZ)
{
  KCHAR *pFirst;
  KCHAR *pLast;
  KCHAR *pVeryLast;
  int i;
  int ilen=(int) strlen(source);



  if(NULL != token) 
  {
      token[0]=0; /* Clear output string. */
  }
  if(ilen < 1) 
  {
     return 0;
  }

  /* Translate any cr's or lf's to delimiters. */
  for(i=0; i<ilen; i++) 
  {
     if( (source[i]=='\r') || (source[i]=='\n') )
     {
	     source[i]=g_delimiters[0];
     }
  }

  pVeryLast=&source[ilen];
  /* Get rid of trailing spaces etc. */
  for(i=ilen-1; i>=0; i--) {
	  KCHAR *pend=strchr(g_delimiters, source[i]);
	  if(pend==source) pFirst=source; /* dummy line for debugging */
	  if( strchr(g_delimiters, source[i])!=NULL ) 
	  {
		  source[i]=0;
		  pVeryLast=&source[i];
	  } else break;
  }

  /* Get rid of initial delimiters. */
  pFirst=source;
  while (*pFirst != '\0') 
  {
	  if(strchr(g_delimiters, (int) *pFirst)==NULL) break;
	  pFirst++;
  }
  
  /* pFirst now points to a char in source which is not a delimiter. */
  
  /* Find end of token. */ 
  pLast=strpbrk( pFirst, g_delimiters );
  if(pLast==0) 
  {
    /* If there was no delimiter, then just use the end of the string. */
    pLast=&pFirst[strlen(pFirst)];
  }



  if(pLast<=pFirst) return NULL;

  /* If the buffer size was specified as zero, then we don't put anything */
  /* in the token output buffer, we simply set the size and then return.  */
  if(NULL!=pnTokenBufSZ)
  {
      if(0==*pnTokenBufSZ)
      {
          *pnTokenBufSZ=(unsigned int) ( pLast-pFirst+1 );
      } else
      {
       /* If *pnTokenBufSZ was NOT zero, then we copy our token into the */
       /* the token buffer, being careful not to write past the end of it. */
       ilen=(int) (pLast-pFirst);
       if(ilen > (int) *pnTokenBufSZ - 1) ilen = (int) *pnTokenBufSZ -1;
       for(i=0; i<ilen; i++)
       {
         token[i]=pFirst[i];
       }

       token[ilen]=0; /* Terminate string in right place. */

       /* If there are no more chars in source, return NULL instead of */
       /* a pointer to the last char in the string. */
       if(pVeryLast<=pLast) return NULL;  
      }
  }
  return pLast;
} 


/****************************************************************************/
/**
 *  Find the first non-space character in a string, and return a pointer to *
 * that character.                                                          *
 *                                                                          *
 ****************************************************************************/
KCHAR *get_ptr_to_first_non_space(KCHAR *chstr)
{
	int ilen;
	KCHAR *pch;
        int i;

	if(chstr==NULL) return NULL;
	ilen=(int) strlen(chstr);
    pch=chstr;
	for(i=0; i<ilen; i++)
	{
		if( *pch!=' ') return pch;
		pch++;
	}
	return NULL;
}


/****************************************************************************/
/**
 *  Find the first character in a string that is not a space nor a linefeed,*
 * and return a pointer to                                                  *
 * that character.                                                          *
 *                                                                          *
 ****************************************************************************/
KCHAR *get_ptr_to_first_non_space_or_lf(KCHAR *chstr)
{
	int ilen;
	KCHAR *pch;
        int i;

	if(chstr==NULL) return NULL;
	ilen=(int) strlen(chstr);
    pch=chstr;
	for(i=0; i<ilen; i++)
	{
		if( (*pch!='\r') && (*pch!=' ') ) return pch;
		pch++;
	}
	return NULL;
}

/**
  * Helper function to print a KCCHAR string.  It might be stored
  * in program flash, so translate the string to a memory based string
  * before trying to print it.
  */
void print_kccstr(KCMD *pKCMD, KCCHAR *pPgmStr)
{
    USER_IO *pUserIO=pKCMD->pUserIO;
    void * conio_context=pKCMD->conio_context;

    int i;
    unsigned int uiLen=(unsigned int) KCC_strlen(pPgmStr);
    KCHAR ch;
    
    for(i=0; i< (int) uiLen; i++)
    {
        ch = KCC_ACCESS(pPgmStr + i);
        pUserIO->pFuncPutc(conio_context, ch, 100);
    }
}

/**
 * kcmd_crlf just types the CR and a LF to the current output.
 */
void kcmd_crlf(KCMD *pKCMD)
{
			   print_kccstr(pKCMD, KCCSTR("\r\n"));
}

