/*
	kcmd command processor - Copyright (C) 2009 Kevin Stokes.

	This file is part of the kcmd command processor distribution
	originally availabe from http://www.boldinventions.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the Eclipse Public License - v 1.0.

	This software has NO WARRANTY.  Use it at your own risk;   It 
	has no warranty of any kind, implied or otherwise.

	There should be a copy of the Eclipse Public License along with
	this source code.  If not, it can be found on the eclipse.org
        website:  http://www.eclipse.org

*/

/** \file kcmd_helper.h has some helper functions for the command processor
  *
  */
#ifndef KCMD_HELPER_H__
#define KCMD_HELPER_H__

/* If we are in a c++ environment, make sure the compiler knows these are */
/* straight C functions. */
#ifdef __cplusplus
extern "C" {
#endif


/**
 * FUNC_CHOUT() is a defined function type for putting out one character
 * to an output device, or a memory buffer. 
 */
typedef void (*FUNC_CHOUT)(void * pContext, KCHAR ch);

/**
 * charBufferCHOUT_t is a structure for queueing up characters in a buffer
 * with a callback function.  This would be the structure used for a context
 * for the callback.
 */
typedef struct struct_charBufferCHOUT
{
    KCHAR *pBuffer;
    unsigned int bufferSize;
    unsigned int bufferIndex;
} charBufferCHOUT_t;


/**
 * print_kccstr simply takes a KCCHAR string (which is in Program
 *    or Flash RAM) and sends it to the output device.  If the
 */
void print_kccstr(KCMD *pKCMD, KCCHAR *pPgmStr);

/**
 * kcmd_crlf just types the CR and a LF to the current output.
 */
void kcmd_crlf(KCMD *pKCMD);

/**
 * parse a character representation of a number and return the integer
 *    or give an error code if conversion was not possible.
 */
int str_to_num(KCHAR * str, int *piErr );

/**
 * parse_text_line breaks tokens out of a string and returns a pointer
 *    to the rest of the command line.
 */
KCHAR *parse_text_line(KCHAR *source, KCHAR *token, unsigned int *pnTokenBufSZ);

/**
 * gets a pointer to first non space character in a string.
 */
KCHAR *get_ptr_to_first_non_space(KCHAR *chstr);

/**
 * gets a pointer to the first character which is 
 * neither a space nor a linefeed character. 
 */
KCHAR *get_ptr_to_first_non_space_or_lf(KCHAR *chstr);


/**
 * int_to_string converts an integer into a decimal (base 10) 
 *   character string representation.
 */ 
void int_to_string(FUNC_CHOUT pFuncCHOUT, void *pContextCHOUT,  
                int flag_leadingzero, int flag_width,  int i_arg);

/**
 * my_sprintf is a partial implementation of sprintf
 */
KCHAR *my_sprintf(KCHAR *buf, KCCHAR *fmt, ...);

/**
 * A printf-like substitute which sends output directly to the
 * user io routines, with no malloc'ing of buffers.
 */
void my_kprintf(KCMD *pKCMD, KCCHAR *fmt, ...);

#define kcmd_Printf_0(pKCMD, kccFormatString) my_kprintf(pKCMD, kccFormatString)
#define kcmd_Printf_1(pKCMD, kccFormatString, arg1) my_kprintf(pKCMD, kccFormatString, arg1)
#define kcmd_Printf_2(pKCMD, kccFormatString, arg1, arg2) my_kprintf(pKCMD, kccFormatString, arg1, arg2)
#define kcmd_Printf_3(pKCMD, kccFormatString, arg1, arg2, arg3) my_kprintf(pKCMD, kccFormatString, arg1, arg2, arg3)
#define kcmd_Printf_4(pKCMD, kccFormatString, arg1, arg2, arg3, arg4) my_kprintf(pKCMD, kccFormatString, arg1, arg2, arg3, arg4)
#define kcmd_Printf_5(pKCMD, kccFormatString, arg1, arg2, arg3, arg4, arg5) my_kprintf(pKCMD, kccFormatString, arg1, arg2, arg3, arg4, arg5)


KINTPTR_t kcmd_stackPop(KCMD *pKCMD);


void kcmd_stackPush(KCMD *pKCMD, int iValue);



void kcmd_stackDup(KCMD *pKCMD);


void kcmd_stackOver(KCMD *pKCMD);


void kcmd_stackSwap(KCMD *pKCMD);


int kcmd_stackAt(KCMD *pKCMD);

int kcmd_stackQuestion(KCMD *pKCMD);

#ifdef __cplusplus
} /* extern "C" ... */
#endif


#endif /* KCMD_HELPER_H__ */
